@@ -110,7 +110,7 @@ JL_DLLEXPORT jl_value_t *jl_eval_string(const char *str)
     JL_TRY {
         const char filename[] = "none";
         jl_value_t *ast = jl_parse_all(str, strlen(str),
-                filename, strlen(filename));
+                filename, strlen(filename), 1);
         JL_GC_PUSH1(&ast);
         r = jl_toplevel_eval_in(jl_main_module, ast);
         JL_GC_POP();
@@ -173,7 +173,7 @@ JL_DLLEXPORT const char *jl_string_ptr(jl_value_t *s)
     return jl_string_data(s);
 }
 
-JL_DLLEXPORT jl_value_t *jl_call(jl_function_t *f, jl_value_t **args, int32_t nargs)
+JL_DLLEXPORT jl_value_t *jl_call(jl_function_t *f, jl_value_t **args, uint32_t nargs)
 {
     jl_value_t *v;
     jl_task_t *ct = jl_current_task;
@@ -411,7 +411,7 @@ JL_DLLEXPORT const char *jl_git_commit(void)
     return commit;
 }
 
-// Create function versions of some useful macros
+// Create function versions of some useful macros for GDB or FFI use
 JL_DLLEXPORT jl_taggedvalue_t *(jl_astaggedvalue)(jl_value_t *v)
 {
     return jl_astaggedvalue(v);
@@ -432,6 +432,11 @@ JL_DLLEXPORT jl_value_t *(jl_get_fieldtypes)(jl_value_t *v)
     return (jl_value_t*)jl_get_fieldtypes((jl_datatype_t*)v);
 }
 
+JL_DLLEXPORT int ijl_egal(jl_value_t *a, jl_value_t *b)
+{
+    return jl_egal(a, b);
+}
+
 
 #ifndef __clang_gcanalyzer__
 JL_DLLEXPORT int8_t (jl_gc_unsafe_enter)(void)
@@ -459,7 +464,7 @@ JL_DLLEXPORT void (jl_gc_safe_leave)(int8_t state)
 }
 #endif
 
-JL_DLLEXPORT void (jl_gc_safepoint)(void)
+JL_DLLEXPORT void jl_gc_safepoint(void)
 {
     jl_task_t *ct = jl_current_task;
     jl_gc_safepoint_(ct->ptls);
@@ -475,6 +480,28 @@ JL_DLLEXPORT void (jl_cpu_wake)(void)
     jl_cpu_wake();
 }
 
+JL_DLLEXPORT void jl_cumulative_compile_timing_enable(void)
+{
+    // Increment the flag to allow reentrant callers to `@time`.
+    jl_atomic_fetch_add(&jl_measure_compile_time_enabled, 1);
+}
+
+JL_DLLEXPORT void jl_cumulative_compile_timing_disable(void)
+{
+    // Decrement the flag when done measuring, allowing other callers to continue measuring.
+    jl_atomic_fetch_add(&jl_measure_compile_time_enabled, -1);
+}
+
+JL_DLLEXPORT uint64_t jl_cumulative_compile_time_ns(void)
+{
+    return jl_atomic_load_relaxed(&jl_cumulative_compile_time);
+}
+
+JL_DLLEXPORT uint64_t jl_cumulative_recompile_time_ns(void)
+{
+    return jl_atomic_load_relaxed(&jl_cumulative_recompile_time);
+}
+
 JL_DLLEXPORT void jl_get_fenv_consts(int *ret)
 {
     ret[0] = FE_INEXACT;
@@ -501,17 +528,6 @@ JL_DLLEXPORT int jl_set_fenv_rounding(int i)
     return fesetround(i);
 }
 
-
-#ifdef JL_ASAN_ENABLED
-JL_DLLEXPORT const char* __asan_default_options()
-{
-    return "allow_user_segv_handler=1:detect_leaks=0";
-    // FIXME: enable LSAN after fixing leaks & defining __lsan_default_suppressions(),
-    //        or defining __lsan_default_options = exitcode=0 once publicly available
-    //        (here and in flisp/flmain.c)
-}
-#endif
-
 static int exec_program(char *program)
 {
     JL_TRY {
@@ -615,8 +631,7 @@ static NOINLINE int true_main(int argc, char *argv[])
 static void lock_low32(void)
 {
 #if defined(_OS_WINDOWS_) && defined(_P64) && defined(JL_DEBUG_BUILD)
-    // Wine currently has a that causes it to answer VirtualQuery incorrectly.
-    // block usage of the 32-bit address space on win64, to catch pointer cast errors
+    // Prevent usage of the 32-bit address space on Win64, to catch pointer cast errors.
     char *const max32addr = (char*)0xffffffffL;
     SYSTEM_INFO info;
     MEMORY_BASIC_INFORMATION meminfo;
@@ -640,11 +655,12 @@ static void lock_low32(void)
                 if ((char*)p != first)
                     // Wine and Windows10 seem to have issues with reporting memory access information correctly
                     // so we sometimes end up with unexpected results - this is just ignore those and continue
-                    // this is just a debugging aid to help find accidental pointer truncation anyways, so it's not critical
+                    // this is just a debugging aid to help find accidental pointer truncation anyways,
+                    // so it is not critical
                     VirtualFree(p, 0, MEM_RELEASE);
             }
         }
-        meminfo.BaseAddress += meminfo.RegionSize;
+        meminfo.BaseAddress = (void*)((char*)meminfo.BaseAddress + meminfo.RegionSize);
     }
 #endif
     return;
@@ -653,16 +669,16 @@ static void lock_low32(void)
 // Actual definition in `ast.c`
 void jl_lisp_prompt(void);
 
-static void rr_detach_teleport(void) {
 #ifdef _OS_LINUX_
+static void rr_detach_teleport(void) {
 #define RR_CALL_BASE 1000
 #define SYS_rrcall_detach_teleport (RR_CALL_BASE + 9)
     int err = syscall(SYS_rrcall_detach_teleport, 0, 0, 0, 0, 0, 0);
     if (err < 0 || jl_running_under_rr(1)) {
         jl_error("Failed to detach from rr session");
     }
-#endif
 }
+#endif
 
 JL_DLLEXPORT int jl_repl_entrypoint(int argc, char *argv[])
 {
@@ -679,16 +695,18 @@ JL_DLLEXPORT int jl_repl_entrypoint(int argc, char *argv[])
         memmove(&argv[1], &argv[2], (argc-2)*sizeof(void*));
         argc--;
     }
-    char **orig_argv = argv;
-    jl_parse_opts(&argc, (char***)&argv);
+    char **new_argv = argv;
+    jl_parse_opts(&argc, (char***)&new_argv);
 
     // The parent process requested that we detach from the rr session.
     // N.B.: In a perfect world, we would only do this for the portion of
     // the execution where we actually need to exclude rr (e.g. because we're
     // testing for the absence of a memory-model-dependent bug).
     if (jl_options.rr_detach && jl_running_under_rr(0)) {
+#ifdef _OS_LINUX_
         rr_detach_teleport();
-        execv("/proc/self/exe", orig_argv);
+        execv("/proc/self/exe", argv);
+#endif
         jl_error("Failed to self-execute");
     }
 
@@ -698,7 +716,7 @@ JL_DLLEXPORT int jl_repl_entrypoint(int argc, char *argv[])
         jl_lisp_prompt();
         return 0;
     }
-    int ret = true_main(argc, (char**)argv);
+    int ret = true_main(argc, (char**)new_argv);
     jl_atexit_hook(ret);
     return ret;
 }
