@@ -2,7 +2,7 @@
 
 module TestSpecial
 
-using Test, LinearAlgebra, SparseArrays, Random
+using Test, LinearAlgebra, Random
 using LinearAlgebra: rmul!
 
 n= 10 #Size of matrix to test
@@ -104,6 +104,28 @@ Random.seed!(1)
             @test LowerTriangular(C) == LowerTriangular(Cdense)
         end
     end
+
+    @testset "Matrix constructor for !isa(zero(T), T)" begin
+        # the following models JuMP.jl's VariableRef and AffExpr, resp.
+        struct TypeWithoutZero end
+        struct TypeWithZero end
+        Base.promote_rule(::Type{TypeWithoutZero}, ::Type{TypeWithZero}) = TypeWithZero
+        Base.convert(::Type{TypeWithZero}, ::TypeWithoutZero) = TypeWithZero()
+        Base.zero(::Type{<:Union{TypeWithoutZero, TypeWithZero}}) = TypeWithZero()
+        LinearAlgebra.symmetric(::TypeWithoutZero, ::Symbol) = TypeWithoutZero()
+        Base.transpose(::TypeWithoutZero) = TypeWithoutZero()
+        d  = fill(TypeWithoutZero(), 3)
+        du = fill(TypeWithoutZero(), 2)
+        dl = fill(TypeWithoutZero(), 2)
+        D  = Diagonal(d)
+        Bu = Bidiagonal(d, du, :U)
+        Bl = Bidiagonal(d, dl, :L)
+        Tri = Tridiagonal(dl, d, du)
+        Sym = SymTridiagonal(d, dl)
+        for M in (D, Bu, Bl, Tri, Sym)
+            @test Matrix(M) == zeros(TypeWithZero, 3, 3)
+        end
+    end
 end
 
 @testset "Binary ops among special types" begin
@@ -188,20 +210,37 @@ end
 
 
 @testset "Triangular Types and QR" begin
-    for typ in [UpperTriangular,LowerTriangular,LinearAlgebra.UnitUpperTriangular,LinearAlgebra.UnitLowerTriangular]
+    for typ in (UpperTriangular, LowerTriangular, UnitUpperTriangular, UnitLowerTriangular)
         a = rand(n,n)
         atri = typ(a)
+        matri = Matrix(atri)
         b = rand(n,n)
-        qrb = qr(b,Val(true))
-        @test *(atri, adjoint(qrb.Q)) ≈ Matrix(atri) * qrb.Q'
-        @test rmul!(copy(atri), adjoint(qrb.Q)) ≈ Matrix(atri) * qrb.Q'
-        qrb = qr(b,Val(false))
-        @test *(atri, adjoint(qrb.Q)) ≈ Matrix(atri) * qrb.Q'
-        @test rmul!(copy(atri), adjoint(qrb.Q)) ≈ Matrix(atri) * qrb.Q'
+        for pivot in (ColumnNorm(), NoPivot())
+            qrb = qr(b, pivot)
+            @test atri * qrb.Q ≈ matri * qrb.Q ≈ rmul!(copy(atri), qrb.Q)
+            @test atri * qrb.Q' ≈ matri * qrb.Q' ≈ rmul!(copy(atri), qrb.Q')
+            @test qrb.Q * atri ≈ qrb.Q * matri ≈ lmul!(qrb.Q, copy(atri))
+            @test qrb.Q' * atri ≈ qrb.Q' * matri ≈ lmul!(qrb.Q', copy(atri))
+        end
+    end
+end
+
+@testset "Multiplication of Qs" begin
+    for pivot in (ColumnNorm(), NoPivot()), A in (rand(5, 3), rand(5, 5), rand(3, 5))
+        Q = qr(A, pivot).Q
+        m = size(A, 1)
+        C = Matrix{Float64}(undef, (m, m))
+        @test Q*Q ≈ (Q*I) * (Q*I) ≈ mul!(C, Q, Q)
+        @test size(Q*Q) == (m, m)
+        @test Q'Q ≈ (Q'*I) * (Q*I) ≈ mul!(C, Q', Q)
+        @test size(Q'Q) == (m, m)
+        @test Q*Q' ≈ (Q*I) * (Q'*I) ≈ mul!(C, Q, Q')
+        @test size(Q*Q') == (m, m)
+        @test Q'Q' ≈ (Q'*I) * (Q'*I) ≈ mul!(C, Q', Q')
+        @test size(Q'Q') == (m, m)
     end
 end
 
-# should all yield sparse arrays
 @testset "concatenations of combinations of special and other matrix types" begin
     N = 4
     # Test concatenating pairwise combinations of special matrices
@@ -211,123 +250,104 @@ end
     symtridiagmat = SymTridiagonal(1:N, 1:(N-1))
     specialmats = (diagmat, bidiagmat, tridiagmat, symtridiagmat)
     for specialmata in specialmats, specialmatb in specialmats
-        @test issparse(hcat(specialmata, specialmatb))
-        @test issparse(vcat(specialmata, specialmatb))
-        @test issparse(hvcat((1,1), specialmata, specialmatb))
-        @test issparse(cat(specialmata, specialmatb; dims=(1,2)))
+        MA = Matrix(specialmata); MB = Matrix(specialmatb)
+        @test hcat(specialmata, specialmatb) == hcat(MA, MB)
+        @test vcat(specialmata, specialmatb) == vcat(MA, MB)
+        @test hvcat((1,1), specialmata, specialmatb) == hvcat((1,1), MA, MB)
+        @test cat(specialmata, specialmatb; dims=(1,2)) == cat(MA, MB; dims=(1,2))
     end
     # Test concatenating pairwise combinations of special matrices with sparse matrices,
     # dense matrices, or dense vectors
     densevec = fill(1., N)
     densemat = diagm(0 => densevec)
-    spmat = spdiagm(0 => densevec)
     for specialmat in specialmats
+        SM = Matrix(specialmat)
         # --> Tests applicable only to pairs of matrices
-        for othermat in (spmat, densemat)
-            @test issparse(vcat(specialmat, othermat))
-            @test issparse(vcat(othermat, specialmat))
-        end
+        @test vcat(specialmat, densemat) == vcat(SM, densemat)
+        @test vcat(densemat, specialmat) == vcat(densemat, SM)
         # --> Tests applicable also to pairs including vectors
-        for specialmat in specialmats, othermatorvec in (spmat, densemat, densevec)
-            @test issparse(hcat(specialmat, othermatorvec))
-            @test issparse(hcat(othermatorvec, specialmat))
-            @test issparse(hvcat((2,), specialmat, othermatorvec))
-            @test issparse(hvcat((2,), othermatorvec, specialmat))
-            @test issparse(cat(specialmat, othermatorvec; dims=(1,2)))
-            @test issparse(cat(othermatorvec, specialmat; dims=(1,2)))
+        for specialmat in specialmats, othermatorvec in (densemat, densevec)
+            SM = Matrix(specialmat); OM = Array(othermatorvec)
+            @test hcat(specialmat, othermatorvec) == hcat(SM, OM)
+            @test hcat(othermatorvec, specialmat) == hcat(OM, SM)
+            @test hvcat((2,), specialmat, othermatorvec) == hvcat((2,), SM, OM)
+            @test hvcat((2,), othermatorvec, specialmat) == hvcat((2,), OM, SM)
+            @test cat(specialmat, othermatorvec; dims=(1,2)) == cat(SM, OM; dims=(1,2))
+            @test cat(othermatorvec, specialmat; dims=(1,2)) == cat(OM, SM; dims=(1,2))
         end
     end
 end
 
-# Test that concatenations of annotated sparse/special matrix types with other matrix
-# types yield sparse arrays, and that the code which effects that does not make concatenations
-# strictly involving un/annotated dense matrices yield sparse arrays
-#
-# TODO: As with the associated code, these tests should be moved to a more appropriate
-# location, particularly some future equivalent of base/linalg/special.jl dedicated to
-# intereactions between a broader set of matrix types
 @testset "concatenations of annotated types" begin
     N = 4
     # The tested annotation types
     testfull = Bool(parse(Int,(get(ENV, "JULIA_TESTFULL", "0"))))
-    utriannotations = (UpperTriangular, LinearAlgebra.UnitUpperTriangular)
-    ltriannotations = (LowerTriangular, LinearAlgebra.UnitLowerTriangular)
+    utriannotations = (UpperTriangular, UnitUpperTriangular)
+    ltriannotations = (LowerTriangular, UnitLowerTriangular)
     triannotations = (utriannotations..., ltriannotations...)
     symannotations = (Symmetric, Hermitian)
     annotations = testfull ? (triannotations..., symannotations...) : (LowerTriangular, Symmetric)
-    # Concatenations involving these types, un/annotated, should yield sparse arrays
-    spvec = spzeros(N)
-    spmat = sparse(1.0I, N, N)
+    # Concatenations involving these types, un/annotated
     diagmat = Diagonal(1:N)
     bidiagmat = Bidiagonal(1:N, 1:(N-1), :U)
     tridiagmat = Tridiagonal(1:(N-1), 1:N, 1:(N-1))
     symtridiagmat = SymTridiagonal(1:N, 1:(N-1))
-    sparseconcatmats = testfull ? (spmat, diagmat, bidiagmat, tridiagmat, symtridiagmat) : (spmat, diagmat)
-    # Concatenations involving strictly these types, un/annotated, should yield dense arrays
+    specialconcatmats = testfull ? (diagmat, bidiagmat, tridiagmat, symtridiagmat) : (diagmat,)
+    # Concatenations involving strictly these types, un/annotated
     densevec = fill(1., N)
     densemat = fill(1., N, N)
     # Annotated collections
     annodmats = [annot(densemat) for annot in annotations]
-    annospcmats = [annot(spcmat) for annot in annotations, spcmat in sparseconcatmats]
-    # Test that concatenations of pairwise combinations of annotated sparse/special
-    # yield sparse matrices
+    annospcmats = [annot(spcmat) for annot in annotations, spcmat in specialconcatmats]
+    # Test concatenations of pairwise combinations of annotated special matrices
     for annospcmata in annospcmats, annospcmatb in annospcmats
-        @test issparse(vcat(annospcmata, annospcmatb))
-        @test issparse(hcat(annospcmata, annospcmatb))
-        @test issparse(hvcat((2,), annospcmata, annospcmatb))
-        @test issparse(cat(annospcmata, annospcmatb; dims=(1,2)))
+        AM = Array(annospcmata); BM = Array(annospcmatb)
+        @test vcat(annospcmata, annospcmatb) == vcat(AM, BM)
+        @test hcat(annospcmata, annospcmatb) == hcat(AM, BM)
+        @test hvcat((2,), annospcmata, annospcmatb) == hvcat((2,), AM, BM)
+        @test cat(annospcmata, annospcmatb; dims=(1,2)) == cat(AM, BM; dims=(1,2))
     end
-    # Test that concatenations of pairwise combinations of annotated sparse/special
-    # matrices and other matrix/vector types yield sparse matrices
+    # Test concatenations of pairwise combinations of annotated special matrices and other matrix/vector types
     for annospcmat in annospcmats
+        AM = Array(annospcmat)
         # --> Tests applicable to pairs including only matrices
-        for othermat in (densemat, annodmats..., sparseconcatmats...)
-            @test issparse(vcat(annospcmat, othermat))
-            @test issparse(vcat(othermat, annospcmat))
+        for othermat in (densemat, annodmats..., specialconcatmats...)
+            OM = Array(othermat)
+            @test vcat(annospcmat, othermat) == vcat(AM, OM)
+            @test vcat(othermat, annospcmat) == vcat(OM, AM)
         end
         # --> Tests applicable to pairs including other vectors or matrices
-        for other in (spvec, densevec, densemat, annodmats..., sparseconcatmats...)
-            @test issparse(hcat(annospcmat, other))
-            @test issparse(hcat(other, annospcmat))
-            @test issparse(hvcat((2,), annospcmat, other))
-            @test issparse(hvcat((2,), other, annospcmat))
-            @test issparse(cat(annospcmat, other; dims=(1,2)))
-            @test issparse(cat(other, annospcmat; dims=(1,2)))
+        for other in (densevec, densemat, annodmats..., specialconcatmats...)
+            OM = Array(other)
+            @test hcat(annospcmat, other) == hcat(AM, OM)
+            @test hcat(other, annospcmat) == hcat(OM, AM)
+            @test hvcat((2,), annospcmat, other) == hvcat((2,), AM, OM)
+            @test hvcat((2,), other, annospcmat) == hvcat((2,), OM, AM)
+            @test cat(annospcmat, other; dims=(1,2)) == cat(AM, OM; dims=(1,2))
+            @test cat(other, annospcmat; dims=(1,2)) == cat(OM, AM; dims=(1,2))
         end
     end
-    # The preceding tests should cover multi-way combinations of those types, but for good
-    # measure test a few multi-way combinations involving those types
-    @test issparse(vcat(spmat, densemat, annospcmats[1], annodmats[2]))
-    @test issparse(vcat(densemat, spmat, annodmats[1], annospcmats[2]))
-    @test issparse(hcat(spvec, annodmats[1], annospcmats[3], densevec, diagmat))
-    @test issparse(hcat(annodmats[2], annospcmats[4], spvec, densevec, diagmat))
-    @test issparse(hvcat((5,), diagmat, densevec, spvec, annodmats[1], annospcmats[1]))
-    @test issparse(hvcat((5,), spvec, annodmats[2], diagmat, densevec, annospcmats[2]))
-    @test issparse(cat(annodmats[1], diagmat, annospcmats[3], densevec, spvec; dims=(1,2)))
-    @test issparse(cat(spvec, diagmat, densevec, annospcmats[4], annodmats[2]; dims=(1,2)))
-    # Test that concatenations strictly involving un/annotated dense matrices/vectors
-    # yield dense arrays
+    # Test concatenations strictly involving un/annotated dense matrices/vectors
     for densemata in (densemat, annodmats...)
+        AM = Array(densemata)
         # --> Tests applicable to pairs including only matrices
         for densematb in (densemat, annodmats...)
-            @test !issparse(vcat(densemata, densematb))
-            @test !issparse(vcat(densematb, densemata))
+            BM = Array(densematb)
+            @test vcat(densemata, densematb) == vcat(AM, BM)
+            @test vcat(densematb, densemata) == vcat(BM, AM)
         end
         # --> Tests applicable to pairs including vectors or matrices
         for otherdense in (densevec, densemat, annodmats...)
-            @test !issparse(hcat(densemata, otherdense))
-            @test !issparse(hcat(otherdense, densemata))
-            @test !issparse(hvcat((2,), densemata, otherdense))
-            @test !issparse(hvcat((2,), otherdense, densemata))
-            @test !issparse(cat(densemata, otherdense; dims=(1,2)))
-            @test !issparse(cat(otherdense, densemata; dims=(1,2)))
+            OM = Array(otherdense)
+            @test hcat(densemata, otherdense) == hcat(AM, OM)
+            @test hcat(otherdense, densemata) == hcat(OM, AM)
+            @test hvcat((2,), densemata, otherdense) == hvcat((2,), AM, OM)
+            @test hvcat((2,), otherdense, densemata) == hvcat((2,), OM, AM)
+            @test cat(densemata, otherdense; dims=(1,2)) == cat(AM, OM; dims=(1,2))
+            @test cat(otherdense, densemata; dims=(1,2)) == cat(OM, AM; dims=(1,2))
         end
     end
 end
-@testset "vcat of Vectors with SparseVectors should yield SparseVector (#22225)" begin
-    @test isa((@inferred vcat(Float64[], spzeros(1))), SparseVector)
-end
-
 
 # for testing types with a dimension
 const BASE_TEST_PATH = joinpath(Sys.BINDIR, "..", "share", "julia", "test")
@@ -395,12 +415,18 @@ using .Main.Furlongs
     @test one(S) isa SymTridiagonal
 
     # eltype with dimensions
-    D = Diagonal{Furlong{2, Int64}}([1, 2, 3, 4])
-    Bu = Bidiagonal{Furlong{2, Int64}}([1, 2, 3, 4], [1, 2, 3], 'U')
-    Bl =  Bidiagonal{Furlong{2, Int64}}([1, 2, 3, 4], [1, 2, 3], 'L')
-    T = Tridiagonal{Furlong{2, Int64}}([1, 2, 3], [1, 2, 3, 4], [1, 2, 3])
-    S = SymTridiagonal{Furlong{2, Int64}}([1, 2, 3, 4], [1, 2, 3])
-    mats = [D, Bu, Bl, T, S]
+    D0 = Diagonal{Furlong{0, Int64}}([1, 2, 3, 4])
+    Bu0 = Bidiagonal{Furlong{0, Int64}}([1, 2, 3, 4], [1, 2, 3], 'U')
+    Bl0 =  Bidiagonal{Furlong{0, Int64}}([1, 2, 3, 4], [1, 2, 3], 'L')
+    T0 = Tridiagonal{Furlong{0, Int64}}([1, 2, 3], [1, 2, 3, 4], [1, 2, 3])
+    S0 = SymTridiagonal{Furlong{0, Int64}}([1, 2, 3, 4], [1, 2, 3])
+    F2 = Furlongs.Furlong{2}(1)
+    D2 = Diagonal{Furlong{2, Int64}}([1, 2, 3, 4].*F2)
+    Bu2 = Bidiagonal{Furlong{2, Int64}}([1, 2, 3, 4].*F2, [1, 2, 3].*F2, 'U')
+    Bl2 =  Bidiagonal{Furlong{2, Int64}}([1, 2, 3, 4].*F2, [1, 2, 3].*F2, 'L')
+    T2 = Tridiagonal{Furlong{2, Int64}}([1, 2, 3].*F2, [1, 2, 3, 4].*F2, [1, 2, 3].*F2)
+    S2 = SymTridiagonal{Furlong{2, Int64}}([1, 2, 3, 4].*F2, [1, 2, 3].*F2)
+    mats = Any[D0, Bu0, Bl0, T0, S0, D2, Bu2, Bl2, T2, S2]
     for A in mats
         @test iszero(zero(A))
         @test isone(one(A))
@@ -434,4 +460,69 @@ end
     end
 end
 
+@testset "BiTriSym*Q' and Q'*BiTriSym" begin
+    dl = [1, 1, 1]
+    d = [1, 1, 1, 1]
+    D = Diagonal(d)
+    Bi = Bidiagonal(d, dl, :L)
+    Tri = Tridiagonal(dl, d, dl)
+    Sym = SymTridiagonal(d, dl)
+    F = qr(ones(4, 1))
+    A = F.Q'
+    for A in (F.Q, F.Q'), B in (D, Bi, Tri, Sym)
+        @test B*A ≈ Matrix(B)*A
+        @test A*B ≈ A*Matrix(B)
+    end
+end
+
+@testset "Ops on SymTridiagonal ev has the same length as dv" begin
+    x = rand(3)
+    y = rand(3)
+    z = rand(2)
+
+    S = SymTridiagonal(x, y)
+    T = Tridiagonal(z, x, z)
+    Bu = Bidiagonal(x, z, :U)
+    Bl = Bidiagonal(x, z, :L)
+
+    Ms = Matrix(S)
+    Mt = Matrix(T)
+    Mbu = Matrix(Bu)
+    Mbl = Matrix(Bl)
+
+    @test S + T ≈ Ms + Mt
+    @test T + S ≈ Mt + Ms
+    @test S + Bu ≈ Ms + Mbu
+    @test Bu + S ≈ Mbu + Ms
+    @test S + Bl ≈ Ms + Mbl
+    @test Bl + S ≈ Mbl + Ms
+end
+
+@testset "Ensure Strided * (Sym)Tridiagonal is Dense" begin
+    x = rand(3)
+    y = rand(3)
+    z = rand(2)
+
+    l = rand(12, 12)
+    # strided but not a Matrix
+    v = @view l[1:4:end, 1:4:end]
+    M_v = Matrix(v)
+    m = rand(3, 3)
+
+    S = SymTridiagonal(x, y)
+    T = Tridiagonal(z, x, z)
+    M_S = Matrix(S)
+    M_T = Matrix(T)
+
+    @test m * T ≈ m * M_T
+    @test m * S ≈ m * M_S
+    @test v * T ≈ M_v * T
+    @test v * S ≈ M_v * S
+
+    @test m * T isa Matrix
+    @test m * S isa Matrix
+    @test v * T isa Matrix
+    @test v * S isa Matrix
+end
+
 end # module TestSpecial
